# Generated by Django 3.1 on 2022-07-16 10:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('telegram_django_bot', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='User',
        ),
    ]
