# Generated by Django 3.1 on 2022-12-02 03:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('telegram_django_bot', '0006_auto_20221023_0157'),
    ]

    operations = [
        migrations.AlterField(
            model_name='botmenuelem',
            name='message_format',
            field=models.CharField(choices=[('T', 'Text'), ('P', 'Image'), ('D', 'Document'), ('A', 'Audio'), ('V', 'Video'), ('G', 'GIF/animation'), ('TV', 'Voice'), ('VN', 'Video note'), ('S', 'Sticker'), ('L', 'Location'), ('GM', 'Media Group')], default='T', max_length=2),
        ),
    ]
